/*
 * locket: to watch for a single ichat user
 * by stephan.com
 * (c) 2005 stephan.com all rights reserved
 */
 
// CUSTOMIZE THE FOLLOWING TWO LINES
var defaultWatchFor = 'StephanLocket'; // The default AIM handle to watch for
var defaultFirstMessage = 'locket: hello'; // The message sent when the locket is clicked on
// END OF CUSTOMIZATION
// YOU SHOULD ALSO CUSTOMIZE THE INFO.PLIST FILE

var watchFor=defaultWatchFor;
var currentStatus = 'unknown';
var openIfAway = false;
var openIfIdle = false;

/*
 * At first glance, the update frequency seems unabashedly slow, but since we ALSO
 * reload every time the user fires up dashboard, it's actually not bad at all
 */
var updateFrequency = 5 * 60000; // every 5 minutes (but only when Dashboard is visible)

var debugEnabled = false;	
var refreshInterval;

// Utility function for debugging.  When in Dashboard, alerts go to console.

function debug(text) {	 
    if (debugEnabled) {	 
        alert(text);	 
    }	 
}

// Go to the site--different if you're testing in a browser vs. in Dashboard.
function gotoURL(url) {
	if (window.widget) {
		widget.openURL(url);
	} else {
		window.location.href = url;
	}
}

// Creates an instance-specific version of a key for persistent
// Dashboard state storage.

function createkey(key)
{
	return widget.identifier + "-" + key;
}

// If Enter key is pressed on the flip side, flip back to front.

function handleEnter() {
    var keyCode = event.keyCode ? event.keyCode : event.which ? event.which : event.charCode;
    if (keyCode == 13) {
        hidePrefs();
    }
}

// Perform the flip from front to back.

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget) {
		window.resizeTo(120,120);
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	} 
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back

	if (window.widget) {

		setTimeout ('widget.performTransition();', 0);	// and flip the widget over	
	}
}


// Perform the flip from back to front, refresh content under the assumption
// that the configuration has changed.

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");


	if (window.widget) {
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	}
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front

	watchFor = document.getElementById("watchForField").value;
	// away/idle checks already set
	
	if (window.widget) {
        widget.setPreferenceForKey(watchFor, createkey("watchFor")); // save value
		widget.setPreferenceForKey(openIfAway, createkey("openIfAway")); 
		widget.setPreferenceForKey(openIfIdle, createkey("openIfIdle")); 
    }
	
	refreshStatus();
	
	if (window.widget) {
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
		window.resizeTo(120,88);
	}

}

/*
 * these functions let you set whether the locket is open when the desired
 * user is "away" or "idle", and update the custom checkmark buttons
 */
 
function toggleAway() {
	openIfAway = !openIfAway;
	updateChecks();
}

function toggleIdle() {
	openIfIdle = !openIfIdle;
	updateChecks();
}

function updateChecks() {
	if(openIfAway) {
		document.getElementById("awayCheck").src = "c/check.png";
	} else {
		document.getElementById("awayCheck").src = "c/uncheck.png";
	}		

	if(openIfIdle) {
		document.getElementById("idleCheck").src = "c/check.png";
	} else {
		document.getElementById("idleCheck").src = "c/uncheck.png";
	}		
}

// startChat also checks status as a side effect
function startChat() {
	if(!window.widget) {
		debug("applescript not supported in Safari");
		currentStatus = "safari error";
		updateDisplay();
		return;
	} else {
		debug("initiating chat");
	}

	if(currentStatus == "available\n") {
		debug("hit on available");
	} else {
		debug("status is not available, it is: \'"+currentStatus+"\'");
	}
	
	if((currentStatus == "available\n") ||
		(openIfAway && currentStatus == "away\n") ||
		(openIfIdle && currentStatus == "idle\n") ) {
		var iChatApplescript = "\
tell application \"iChat\"\n\
	send \"locket: hello\" to (accounts where handle is \""+watchFor+"\")\n\
	status of (accounts where handle is \""+watchFor+"\") as string\n\
end tell";

		var script = "/usr/bin/osascript -e '"+iChatApplescript+"'";

	currentStatus = widget.system(script,null).outputString;
	debug("result from startchat= "+currentStatus);
	updateDisplay();
	}
}

function refreshStatus() {
	if(!window.widget) {
		debug("applescript not supported in Safari");
		currentStatus = "safari error";
		updateDisplay();
		return;
	} else {
		debug("checking status");
	}
	
	var iChatApplescript = "\
tell application \"iChat\"\n\
	status of (accounts where handle is \""+watchFor+"\") as string\n\
end tell";
 
	var script = "/usr/bin/osascript -e '"+iChatApplescript+"'";

	currentStatus = widget.system(script,null).outputString;
	debug("result = "+currentStatus);
	updateDisplay();
}

// Update the HTML content for the currentStatus based on the currentStatus variable.

function updateDisplay() {
    if (window.widget) {	 
        debug('updating display to reflect ' + currentStatus + ' currentStatus');	 
    }
    debug("update");

	if( (currentStatus == "available\n") ||
		(openIfAway && currentStatus == "away\n") ||
		(openIfIdle && currentStatus == "idle\n") ) {
		document.getElementById("theLocket").src="l/open.png";
	} else {
		document.getElementById("theLocket").src="l/closed.png";
	}
			
}

// We don't want the widget refreshing when it's not on the screen, so these
// functions start and stop the refresh timer.

function setRefreshTimer() {	 
    if (refreshInterval == null) {	 
//        debug("setting refresh timer");	 
        refreshInterval = setInterval("refreshStatus();", updateFrequency);	 
    }	 
}	 
 	 
function clearRefreshTimer() {	 
    if (refreshInterval != null) {	 
        debug("clearing refresh timer");	 
        clearInterval(refreshInterval);	 
        refreshInterval = null;
    }
}
              
function init() {
    // Attempt to retrieve saved configuration.
    if (window.widget) {
        watchFor = widget.preferenceForKey(createkey("watchFor"));
        if (!watchFor || (watchFor == "undefined")) {
        	watchFor = defaultWatchFor;
        }
        openIfAway = widget.preferenceForKey(createkey("openIfAway"));
        if (!openIfAway || (openIfAway == "undefined")) {
        	openIfAway = false;
        }
        openIfIdle = widget.preferenceForKey(createkey("openIfIdle"));
        if (!openIfIdle || (openIfIdle == "undefined")) {
        	openIfIdle = false;
        }
    }
    
    updateDisplay();

	txt = document.getElementById("watchForField");
	txt.value = watchFor;
    updateChecks();


	if (window.widget) {	 
        widget.onshow = function OnShow() { // when Dashboard triggered	 
            debug("widget shown");	 
            refreshStatus();	 
            setRefreshTimer();	 
        }	 
        widget.onremove = function OnRemove() { // when widget discarded 
            debug("widget removed");
            widget.setPreferenceForKey(null, createkey("watchFor"));	 
            widget.setPreferenceForKey(null, createkey("openIfAway"));	 
            widget.setPreferenceForKey(null, createkey("openIfIdle"));	 
            clearRefreshTimer();	 
        }	 
        widget.onhide = function OnHide() { // when Dashboard dismissed	 
            debug("widget hidden");	 
            clearRefreshTimer();	 
        }	 
    }	 
 	
    setRefreshTimer();

}

// boilerplate code for infobutton animation taken from Apple

var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0,
				firstElement:null, timer:null};

function mousemove (event) {
	if (!flipShown) {
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer = null;
		}

		var starttime = (new Date).getTime() - 13;
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit (event) {
	if (flipShown) {
		// fade in the info button
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
	T = limit_3(time-animation.starttime, 0, animation.duration);
	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	} else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
	return from + (to - from) * ease;
}